# Introduction to R
# September 11/12, 2013
# StatLab@UVa Library
# Clay Ford

# Preface comments with '#'

log(27) + 3 # R as a calculator
x <- log(27) + 3 # assign results of calculation to x
# functions often have extra arguments (here, 'base=')
y <- log(1000,base=10) 
x; y # view x and y
rm(x, y) # remove x and y

# Show example of incomplete command
# the prompt turns to a '+'
# To fix, either complete the command or hit ESC

# get help on the log function
help(log)
??"log" # search help for string "log"
# if help topic comes with examples, run them with examples()
example(log)
rm(x) # remove the variable created from the example

# The working directory
getwd() # shows your working directory
# unless you specify otherwise, R will look for input here and/or send
# output there.
# Set working directory in R Studio: Session...Set Working Directory.
# or use setwd() function;
# must use the forward slash / or double backslash \\ in R!
# setwd("C:/Users/jcf2d/Documents/clients/")
    
# clear the console: Ctrl + L

##############
#PRELIMINARIES
##############

# vectors
vals1 <- 22:26 # sequence of integers
vals1
vals2 <- c(22,23,24,25,26) # combine function
vals2
vals3 <- seq(22,26,by=1) # generate sequence of numbers
vals3
# these basically do the same thing
# types are different: numeric vs. integer


# indexing vectors using []
vals1
vals1[1] # show first element 
vals1[-1] # show all but first
vals1[vals1 > 2] # show elements greater than 22
vals1[vals1 > 22 & vals1 <= 25] # greater than 22 and less than equal to 25
vals1[c(2,4)] # show 2nd and 4th elements

# operations on vectors
vals1
vals1 + 100
vals1 # notice original vector has not changed
vals1^2 # square each value
vals1 + vals2 + vals3 # add all 3 vectors
rbind(vals1,vals2,vals3) # combine the rows to form a matrix
all <- rbind(vals1,vals2,vals3) # save the matrix
all
rm(vals2,vals3,all)

# data types (modes): integer, numeric, logical, character, factor
# numeric and integer: see above
# logical (True/False)
vals1
vals1 > 22 # returns vector of TRUE/FALSE
results <- vals1 > 22 # save the vector of T/F
results # view vector
sum(results) # TRUE = 1, FALSE = 0, so sum is count of TRUE elements
sum(results)/length(results) # proportion of vals1 greater than 2
rm(vals1, results)

# character
words <- c("thanks", "for", "coming","today","!")
words
substr(words,1,3) # selects first through third letters
nchar(words)
nchar(words)<6 # logical vector
which(nchar(words)<6) # returns position of words < 6 characters
words[which(nchar(words)<6)] # returns words < 6 characters
rm(words)

#factor (ie, categorical variable)
data <- c(1,2,2,3,1,2,3,3,1,2,3,3,1)
data
fdata <- factor(data)
fdata
class(data)
class(fdata)
summary(data) # descriptive stats
summary(fdata) # counts
fdata <- factor(data, labels=c("I","II","III")) # add labels
summary(fdata)
rm(fdata,data)


###################################
# ExTENDED 'REAL LIFE' EXAMPLE
###################################

# Forbes 2000 list from 2004 
# source: Handbook of Statistical Analysis Using R (Everitt & Hothorn)
    
# Import a CSV fileand create a "dataframe"; here are three ways to do it:
# (1) read file from local directory on computer
forbes <- read.csv("workshop/data/Forbes2000.csv",header=TRUE)
# (2) read from web site 
forbes <- read.csv("http://people.virginia.edu/~jcf2d/workshops/R/Forbes2000.csv",
                       header=TRUE)
# (3) use the R Studio "Import Dataset" button in the workspace window

# inspecting your data 
class(forbes) # tells me this is a data frame
forbes # not terribly useful

head(forbes) # first 6 records
tail(forbes) # last 6 records
summary(forbes) # summary of variables
str(forbes) # view structure of data.frame; very useful function!
forbes$name <- as.character(forbes$name) # change name to character type
str(forbes) # view structure of data.frame
dim(forbes) # dimensions of data frame
nrow(forbes)
ncol(forbes)
names(forbes) # names of columns
# change a column name
names(forbes)[2] <- "company"
names(forbes)

# indexing with data frames [rows,columns]
# show first 6 records and first 2 columns
forbes[1:6,1:2] # [rows, columns]
forbes[1:6,c("rank","company")]

# show last 6 records and company and profits columns
forbes[1995:2000,c("rank","company")]
# same as above but more general
forbes[(nrow(forbes)-5):nrow(forbes),c("rank","company")]

# see the levels of 'category' (a factor)
levels(forbes[,"category"])
levels(forbes$category) # same thing

# calculate frequencies of category
table(forbes$category)
summary(forbes$category)

# summarise numeric columns
mean(forbes$sales)
median(forbes$sales)
range(forbes$sales) # returns min and max values
quantile(forbes$sales)
quantile(forbes$sales, probs=c(0.1,0.9)) # 10th and 90th quantiles
summary(forbes$sales)


# some data manipulation
# create a vector of just the companies
Companies <- forbes[,"company"]
Companies <- forbes$company # same thing

# extract two columns and make a new data frame
Sales <- forbes[,c("company","sales")]
head(Sales)
# two handy tricks
table(Sales$sales > 100) # how many companies with sales > $100B
Sales[Sales$sales > 100,] # list companies with sales > $100B

# create new data frame for companies with sales > $100B
top.sales <- Sales[Sales$sales > 100,] 
top.sales
rownames(top.sales) <- NULL # reset row numbers
top.sales
# order top.sales by sales
order(top.sales$sales) # returns indices of ordered elements (ascending)
top.sales[order(top.sales$sales),] # shows ordered data frame
top.sales[order(top.sales$sales,
                decreasing=TRUE),] # in descending order

# finding missing data
complete.cases(forbes) # returns logical vector (TRUE = no missing data)
table(complete.cases(forbes)) # how many records have missing data
which(complete.cases(forbes)==FALSE) # which records (indices)
which(!complete.cases(forbes)) # another way using '!' (means 'not')
miss <- which(complete.cases(forbes)==FALSE) # save indices (row numbers)
forbes[miss,] # subset data using indices and see the companies
# we could remove those records if we wanted:
# forbes2 <- forbes[-miss,]

# Subset the data with subset function
# create data frame of just Japanese companies and drop company column
JapanComp <- subset(forbes, country == "Japan", 
                 select = -country)

# save a data frame for later use
save(JapanComp,file="JapanComp.Rda") # saved to your working directory
rm(JapanComp)
# load the saved data frame
load("JapanComp.Rda")

# add new column to data frame (marketvalue centered on mean)
forbes$cmv <- forbes$marketvalue - mean(forbes$marketvalue)
head(forbes)
tail(forbes)
# remove column
forbes$cmv <- NULL

# create categories (factors) from continuous variables 
# sales catgeories based on 0.25, 0.50 and 0.75 quantiles
forbes$salesF <- cut(forbes$sales,
              breaks=quantile(forbes$sales),
              labels=c("0-25","26-50","51-75","76-100"))

# create a cross tab of categories vs. sales factor
table(forbes$category, forbes$salesF)


# calculate some descriptive stats
# find median profits by category
aggregate(profits ~ category, data=forbes, median) # returns data frame
# another way using an 'apply' function
# syntax: tapply(vector,grouping variable,function)
tapply(forbes$profits, forbes$category, median) # returns NA values!
tapply(forbes$profits, forbes$category, median, na.rm=TRUE) # ignore missing

# using 'apply' function to find std deviations of sales and profits
# syntax: apply(data.frame, margin, function); 1=rows, 2=columns
apply(forbes[,c("sales","profits")],2,sd) # notice the NA for profits
apply(forbes[,c("sales","profits")],2,sd,na.rm=TRUE) # ignore missing

# some simple graphics
# histograms
hist(forbes$marketvalue) # skew
hist(log(forbes$marketvalue)) # more symmetric
hist(log(forbes$marketvalue),prob=TRUE) # show probability densities
hist(log(forbes$marketvalue),prob=TRUE, breaks=20) # more bins


# Phrases like "forbes$marketvalue" get tedious to type
# 'Attaching' the data frame can make life easier
attach(forbes) # adds data frame to search path
search()
hist(log(marketvalue), prob=TRUE, main="Log Transformed Market Value")

# stack the histograms and make pretty
par(mfrow=c(2,1)) # divide graphics window into 2 "frames"
hist(marketvalue, main="Market Value", col="red")
hist(log(marketvalue), main="Log Market Value", col="blue")

# Do 4 histograms at once
par(mfrow=c(2,2)) # set the graphics device to display 2 x 2 grid
hist(forbes[,5])
hist(forbes[,6])
hist(forbes[,7])
hist(forbes[,8])

# use a "for loop" to do 4 at a time
for(i in 5:8) hist(forbes[,i]) 

# make it look nicer with header and x-axis labels
names(forbes)
for(i in 5:8) {
  hist(forbes[,i], 
       main = paste("Histogram of" , names(forbes)[i]),
       xlab = names(forbes)[i])
}
# reset graphics window to 1x1 frame
par(mfrow=c(1,1))

# boxplots on subset of data
boxplot(log(marketvalue) ~ country, 
        data = subset(forbes, 
                      country %in% c("United Kingdom","Germany","India","Turkey")), 
        ylab="log(market value)")
# the subset() function carries over unused factor levels, 
# therefore we need to use the droplevels() function on country
boxplot(log(marketvalue) ~ droplevels(country), 
        data = subset(forbes, 
                      country %in% c("United Kingdom","Germany","India","Turkey")), 
        ylab="log(market value)")

# some statisical analysis
# basic linear regression
plot(marketvalue ~ sales, pch=5,col="red") # ripe for transformation
plot(log(marketvalue) ~ log(sales), pch=5,col="red")

# do simple linear regression: lm() function
# Do sales explain marketvalue?
mod <- lm(log(marketvalue) ~ log(sales), data=forbes) # save results
mod # see formula and coefficients
summary(mod) # see more
confint(mod) # 95% confidence interval for model coefficients
abline(mod) # add the regression line to the plot

# interpreting slope when both variables log transformed:
# for each 1% difference in sales, 
# the predicted difference in marketvalue is 0.5% (among the Forbes 2000)

# check assumptions with diagnostic plots
par(mfrow=c(2,2)) # divide the plot area into 4 boxes (multi-frame)
plot(mod) # diagnostic plots
par(mfrow=c(1,1)) # return to default


rm(list=ls()) # tidy up

###################################
# PACKAGES
###################################
# Example: To import a Stata file, use the read.dta() function
# need to use 'foreign' package for this function
# foreign comes with base installation
help(read.dta) # not found; need to load foreign package
search() # which packages are attached
library() # see list of installed packages
library(foreign) # this package provides function to import Stata files (and many others)
help(read.dta)
# import data on underage drinking from UCLA Stats consulting site:
alcohol <- read.dta("http://www.ats.ucla.edu/stat/stata/examples/alda/data/alcohol1_pp.dta")

# To install packages in R Studio:
#   1. click Install Packages
#   2. enter name of package in Package field
#   3. click Install

# via code: install.packages("name of package")

# Two packages of interest: plyr and ggplot2 (by Hadley Wickham)
# plyr:    Tools for splitting, applying and combining data.
#          http://plyr.had.co.nz/
# ggplot2: powerful graphics language for creating elegant plots
#          http://ggplot2.org/

# Note: Packages often have dependencies. This means installing one package
# will also install other packages it depends on. Example: installing
# ggplot2 package will install 9 other packages it uses.


###################################
# MORE ANALYSIS EXAMPLES
###################################
# 1991 General Social Survey
# An Introduction to Categorical Data Analysis (Agresti, 1996), p. 31
# Table 2.5

# manually enter data from Table 2.5
table_2.5 <- matrix(c(279,73,225,165,47,191),nrow=2, byrow=TRUE)
table_2.5
colnames(table_2.5) <- c("Dem","Ind","Rep")
rownames(table_2.5) <- c("Females","Males")
table_2.5

# chi square test of independence
chisq.test(table_2.5)
results <- chisq.test(table_2.5)
str(results) # more than meets the eye
results$p.value
results$expected
rm(table_2.5, results)

# t test using data that comes with R (see datsets package)
# Student's sleep data
# Data which show the effect of two soporific drugs 
# (increase in hours of sleep compared to control) on 10 patients.
sleep
plot(extra ~ group, data = sleep)

# one way to do a t test
t.test(sleep$extra[sleep$group == 1], sleep$extra[sleep$group == 2])
# same with less typing using with() function
with(sleep, t.test(extra[group == 1], extra[group == 2]))
# even less typing with the Formula interface
t.test(extra ~ group, data = sleep)
t.test(extra ~ group, data = sleep, var.equal = TRUE) # assume equal variance



###################################
# simulation example
###################################
# generate a normally distributed population
pop <- rnorm(10000, mean = 140, sd = 15)
hist(pop)
hist(pop, prob=TRUE, ylim=c(0,0.03))
x <- 80:200
lines(x, dnorm(x, mean=140, sd=15)) # add normal curve for N(140,15)
# take a sample from the population
sample(pop,20, replace=TRUE)

# take a 1000 samples and calculate mean of each sample
# sampling distribution of the mean
m <- rep(NA,1000) # an empty vector to store means
for (i in 1:1000){
  m[i] <- mean(sample(pop,20,replace=TRUE))
}

mean(pop); mean(m) # compare means
sd(pop); sd(m) # compare standard deviations
15/sqrt(20) # theoretical SD of sampling distribution

# compare histograms
par(mfrow=c(2,1)) # split plot area into 2 rows
# pop'n histogram
hist(pop, freq=FALSE, xlim=c(80,200), ylim=c(0,0.15))
lines(x, dnorm(x, mean=140, sd=15))
# sampling distribution histogram
hist(m, freq=FALSE, xlim=c(80,200), ylim=c(0,0.15))
lines(x, dnorm(x, mean=140, sd=(15/sqrt(20))))
par(mfrow=c(1,1))
rm(i,m,pop,x)

###################################
# function and programming example
###################################
# a function that tells you if x is evenly divisible by y
divisible <- function(x,y){
  
  # error check; return a special message
  if (is.numeric(c(x,y))==FALSE) 
    stop("please enter numbers")
  
  if (x %% y == 0){
    print("yes")
  } else {
    print("no")
  }
}

# test function
divisible(245,23)
# notice x and y are NOT in workspace; they are 'local' to the function

n1 <- 1589
n2 <- 19
divisible(n1,n2)
rm(n1,n2)

